# -*- perl -*-

#
# Author: Slaven Rezic
#
# Copyright (C) 2016,2017,2021 Slaven Rezic. All rights reserved.
# This package is free software; you can redistribute it and/or
# modify it under the same terms as Perl itself.
#
# Mail: slaven@rezic.de
# WWW:  http://www.rezic.de/eserte/
#

package App::orgdaemon;

use strict;
our $VERSION = '1.11';

1;

__END__

=head1 NAME

App::orgdaemon - watch for appointments in org-mode files

=head1 DESCRIPTION

See L<org-daemon>. The module L<App::orgdaemon> is just a placeholder for CPAN.

=head1 SEE ALSO

L<org-daemon>.

=cut
